package org.squarebrackets;

import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.Objects;
import java.util.logging.Logger;

/**
 * Inspired by JDK 8 Tripwire class.
 */
final class Tripwire {

    /** org.squarebrackets Package. */
    private static final Package PACKAGE = Tripwire.class.getPackage();

    /** System property. */
    private static final String TRIPWIRE_PROPERTY = "org.squarebrackets.tripwire";

    /** Should debugging checks be enabled? */
    static final boolean ENABLED = AccessController.doPrivileged(
            (PrivilegedAction<Boolean>) () -> Boolean.getBoolean(TRIPWIRE_PROPERTY));

    /**
     * Class cannot be instantiated.
     */
    private Tripwire() {
    }

    /**
     * Produces a log warning.
     */
    static void trip(Class<?> trippingClass) {
        String msg = "Primitive boxing or unboxing detected at: %s.";
        StackTraceElement[] stack = Thread.currentThread().getStackTrace();
        StackTraceElement element;
        int index = 3;
        do {
            element = stack[index];
            try {
                if (!Objects.equals(PACKAGE, Class.forName(element.getClassName()).getPackage())) {
                    break;
                }
            } catch (Throwable ignored) {
                break;
            }
        } while (++index < stack.length);
        Logger.getLogger(trippingClass.getName()).warning(String.format(msg, element));
    }
}